<?php
require_once dirname(__FILE__) . DS . 'apis' . DS . 'convoy_api.php';

/**
 * Covnoy Automatic Stock plugin handler
 *
 * @link https://aleksa.tf Aleksa Đorđić
 * @link https://code-cats.com Code Cats
 */
class ConvoyAutomaticStockPlugin extends Plugin
{
    public function __construct()
    {
        // Load components required by this plugin
        Loader::loadComponents($this, ['Input', 'Record']);

        Language::loadLang('convoy_automatic_stock_plugin', null, dirname(__FILE__) . DS . 'language' . DS);
        $this->loadConfig(dirname(__FILE__) . DS . 'config.json');
    }

    /**
     * Performs any necessary bootstraping actions
     *
     * @param int $plugin_id The ID of the plugin being installed
     */
    public function install($plugin_id)
    {
        try {
            // Create database tables
            // convoy_stock_settings
            $this->Record
                ->setField(
                    'company_id',
                    [
                        'type' => 'INT',
                        'size' => "10",
                        'unsigned' => true,
                    ]
                )
                ->setKey(['company_id'], 'primary')
                ->setField(
                    'enabled',
                    [
                        'type' => 'TINYINT',
                        'size' => "1",
                    ]
                )
                ->setField(
                    'convoy_module_id',
                    [
                        'type' => 'BIGINT',
                        'size' => "10",
                    ]
                )
                ->create('convoy_stock_settings', true);
                
        } catch (Exception $e) {
            // Error adding... no permission?
            $this->Input->setErrors(['db' => ['create' => $e->getMessage()]]);
            return;
        }

        // Add cron tasks for this plugin
        $this->addCronTasks($this->getCronTasks());
    }

    /**
     * Performs any necessary cleanup actions
     *
     * @param int $plugin_id The ID of the plugin being uninstalled
     * @param bool $last_instance True if $plugin_id is the last instance across
     *  all companies for this plugin, false otherwise
     */
    public function uninstall($plugin_id, $last_instance)
    {
        Loader::loadModels($this, ['CronTasks']);

        // Fetch the cron tasks for this plugin
        $cron_tasks = $this->getCronTasks();

        if ($last_instance) {
            try {
                // Remove database tables
                $this->Record->drop('convoy_stock_settings');
            } catch (Exception $e) {
                // Error dropping... no permission?
                $this->Input->setErrors(['db' => ['create' => $e->getMessage()]]);
                return;
            }

            // Remove the cron tasks
            foreach ($cron_tasks as $task) {
                $cron_task = $this->CronTasks->getByKey($task['key'], $task['dir'], $task['task_type']);
                if ($cron_task) {
                    $this->CronTasks->deleteTask($cron_task->id, $task['task_type'], $task['dir']);
                }
            }
        }

        // Remove individual cron task runs
        foreach ($cron_tasks as $task) {
            $cron_task_run = $this->CronTasks->getTaskRunByKey($task['key'], $task['dir'], false, $task['task_type']);
            if ($cron_task_run) {
                $this->CronTasks->deleteTaskRun($cron_task_run->task_run_id);
            }
        }
    }

    /**
     * Attempts to add new cron tasks for this plugin
     *
     * @param array $tasks A list of cron tasks to add
     */
    private function addCronTasks(array $tasks)
    {
        Loader::loadModels($this, ['CronTasks']);
        foreach ($tasks as $task) {
            $task_id = $this->CronTasks->add($task);

            if (!$task_id) {
                $cron_task = $this->CronTasks->getByKey($task['key'], $task['dir'], $task['task_type']);
                if ($cron_task) {
                    $task_id = $cron_task->id;
                }
            }

            if ($task_id) {
                $task_vars = ['enabled' => $task['enabled']];
                if ($task['type'] === 'time') {
                    $task_vars['time'] = $task['type_value'];
                } else {
                    $task_vars['interval'] = $task['type_value'];
                }

                $this->CronTasks->addTaskRun($task_id, $task_vars);
            }
        }
    }

    /**
     * Retrieves cron tasks available to this plugin along with their default values
     *
     * @return array A list of cron tasks
     */
    private function getCronTasks()
    {
        return [
            [
                'key' => 'update_stock',
                'task_type' => 'plugin',
                'dir' => 'convoy_automatic_stock',
                'name' => Language::_('ConvoyAutomaticStockPlugin.getCronTasks.update_stock', true),
                'description' => Language::_('ConvoyAutomaticStockPlugin.getCronTasks.update_stock_description', true),
                'type' => 'interval',
                'type_value' => '5',
                'enabled' => 1
            ]
        ];
    }

    /**
     * Runs the cron task identified by the key used to create the cron task
     *
     * @param string $key The key used to create the cron task
     * @see CronTasks::add()
     */
    public function cron($key)
    {
        switch ($key) {
            case 'update_stock':
                $this->updateStock();
                break;
        }
    }

    public function updateStock()
    {
        $company_id = Configure::get('Blesta.company_id');
        Loader::loadModels($this, ['ConvoyAutomaticStock.ConvoyStockSettings']);
        if (!($settings = $this->ConvoyStockSettings->get($company_id)) ||
            $settings->enabled != '1') {
            return;
        }

        Loader::loadModels($this, ['ModuleManager', 'Packages']);
        if ($settings->convoy_module_id >= 0) {
            $convoy_module = $this->ModuleManager->get($settings->convoy_module_id);
            $this->processModule($convoy_module);
        }
    }

    private function processModule($module)
    {
        $rows = $module->rows;
        $packages = $this->Packages->getAll(
            Configure::get('Blesta.company_id'), 
            ['name' => 'ASC'], 
            null, 
            null, 
            ['module_id' => $module->id]
        );

        foreach ($rows as $row) {
            $api = $this->getApi(
                $row->meta->secret,
                $row->meta->url
            );

            $response = $api->getNodes();
            if ($response->status() != '200') {
                continue;
            }

            foreach ($packages as $package) {
                $this->applyLimitsToPackage($api, $this->Packages->get($package->id));
            }
        }
    }

    /**
     * Initializes the API and returns an instance of that object with the given $host and $api_key set
     */
    private function getApi($secret, $url)
    {
        return new ConvoyApi($secret, $url);
    }
    
    private function applyLimitsToPackage(ConvoyApi $api, $package)
    {
        $meta = $package->meta;

        $node_ids = $package->meta->nodes;

        
        $possible_quantity = 0;
        foreach ($node_ids as $node_id) {
            $response = $api->getNode($node_id);
            if ($response->status() != '200') {
                continue;
            }
            
            $node = $response->response()->data;

            // Apply overallocation
            $memory = $node->memory * ((100 + $node->memory_overallocate) / 100); 

            // Calcualte free resources
            $memory_free = $memory - $node->memory_allocated;

            $possible_quantity += floor($memory_free / ($meta->memory * 1024 * 1024 * 1024));
        }
        
        $this->Packages->edit($package->id, [
            'module_id' => $package->module_id,
            'module_row' => $package->module_row,
            'module_group' => $package->module_group,
            'meta' => json_decode(json_encode($package->meta), true),
            'qty' => $possible_quantity
        ]);
    }
}
