<?php
/**
 * convoy_stock_settings Management
 *
 * @link https://aleksa.tf Aleksa Đorđić
 * @link https://code-cats.com Code Cats
 */
class ConvoyStockSettings extends ConvoyAutomaticStockModel
{
    /**
     * Returns a list of records for the given company
     *
     * @param array $filters A list of filters for the query
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @param int $page The page number of results to fetch
     * @param array $order A key/value pair array of fields to order the results by
     * @return array An array of stdClass objects
     */
    public function getList(
        array $filters = [],
        $page = 1,
        array $order = ['company_id' => 'desc']
    ) {
        $records = $this->getRecord($filters)
            ->order($order)
            ->limit($this->getPerPage(), (max(1, $page) - 1) * $this->getPerPage())
            ->fetchAll();

        return $records;
    }

    /**
     * Returns the total number of record for the given filters
     *
     * @param array $filters A list of filters for the query
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @return int The total number of records for the given filters
     */
    public function getListCount(array $filters = [])
    {
        return $this->getRecord($filters)->numResults();
    }

    /**
     * Returns all records in the system for the given filters
     *
     * @param array $filters A list of filters for the query
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @param array $order A key/value pair array of fields to order the results by
     * @return array An array of stdClass objects
     */
    public function getAll(
        array $filters = [],
        array $order = ['company_id' => 'desc']
    ) {
        $records = $this->getRecord($filters)->order($order)->fetchAll();

        return $records;
    }

    /**
     * Fetches the record with the given identifier
     *
     * @param int $company_id The identifier of the record to fetch
     * @return mixed A stdClass object representing the record, false if no such record exists
     */
    public function get($company_id)
    {
        $record = $this->getRecord(['company_id' => $company_id])->fetch();

        return $record;
    }

    /**
     * Add a record
     *
     * @param array $vars An array of input data including:
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @return int The identifier of the record that was created, void on error
     */
    public function add(array $vars)
    {
        $this->Input->setRules($this->getRules($vars));

        if ($this->Input->validates($vars)) {
            $fields = ['company_id','enabled','convoy_module_id'];
            $this->Record->insert('convoy_stock_settings', $vars, $fields);

            return $this->Record->lastInsertId();
        }
    }

    /**
     * Edit a record
     *
     * @param int $company_id The identifier of the record to edit
     * @param array $vars An array of input data including:
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @return int The identifier of the record that was updated, void on error
     */
    public function edit($company_id, array $vars)
    {
        
        $vars['company_id'] = $company_id;
        $this->Input->setRules($this->getRules($vars, true));

        if ($this->Input->validates($vars)) {
            $fields = ['company_id','enabled','convoy_module_id'];
            $this->Record->where('company_id', '=', $company_id)->update('convoy_stock_settings', $vars, $fields);

            return $company_id;
        }
    }

    /**
     * Permanently deletes the given record
     *
     * @param int $company_id The identifier of the record to delete
     */
    public function delete($company_id)
    {
        // Delete a record
        $this->Record->from('convoy_stock_settings')->
            where('convoy_stock_settings.company_id', '=', $company_id)->
            delete();
    }

    /**
     * Returns a partial query
     *
     * @param array $filters A list of filters for the query
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @return Record A partially built query
     */
    private function getRecord(array $filters = [])
    {
        $this->Record->select()->from('convoy_stock_settings');

        if (isset($filters['company_id'])) {
            $this->Record->where('convoy_stock_settings.company_id', '=', $filters['company_id']);
        }

        if (isset($filters['enabled'])) {
            $this->Record->where('convoy_stock_settings.enabled', '=', $filters['enabled']);
        }

        if (isset($filters['convoy_module_id'])) {
            $this->Record->where('convoy_stock_settings.convoy_module_id', '=', $filters['convoy_module_id']);
        }

        return $this->Record;
    }

    /**
     * Returns all validation rules for adding/editing extensions
     *
     * @param array $vars An array of input key/value pairs
     *
     *  - company_id
     *  - enabled
     *  - convoy_module_id
     * @param bool $edit True if this if an edit, false otherwise
     * @return array An array of validation rules
     */
    private function getRules(array $vars, $edit = false)
    {
        $rules = [
            'company_id' => [
                'valid' => [
                    'if_set' => $edit,
                    'rule' => true,
                    'message' => Language::_('ConvoyStockSettings.!error.company_id.valid', true)
                ]
            ],
            'enabled' => [
                'valid' => [
                    'if_set' => $edit,
                    'rule' => true,
                    'message' => Language::_('ConvoyStockSettings.!error.enabled.valid', true)
                ]
            ],
            'convoy_module_id' => [
                'valid' => [
                    'if_set' => $edit,
                    'rule' => true,
                    'message' => Language::_('ConvoyStockSettings.!error.convoy_module_id.valid', true)
                ]
            ]
        ];

        return $rules;
    }
}
